<?php declare(strict_types=1);
/**
 * Model: Represents a menu
 */

namespace Infomaker\Everyware\Base\Models;

use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Fluent;
use Infomaker\Everyware\Support\Interfaces\Arrayable;
use Infomaker\Everyware\Support\Interfaces\Jsonable;
use WP_Term;

/**
 * Menu
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base\Models
 * @since   Infomaker\Everyware\Base\Models\Menu 1.0.0
 */
class Menu extends Fluent
{

    /**
     * Contains the defaults for fetching menus via WP
     *
     * @var array
     */
    protected static $defaults = [
        'taxonomy' => 'nav_menu',
        'orderby' => 'name',
        'order' => 'ASC',
        'hide_empty' => false,
        'include' => [],
        'exclude' => [],
        'exclude_tree' => [],
        'number' => '',
        'offset' => '',
        'fields' => 'all',
        'name' => '',
        'slug' => '',
        'hierarchical' => true,
        'search' => '',
        'name__like' => '',
        'description__like' => '',
        'pad_counts' => false,
        'get' => '',
        'child_of' => 0,
        'parent' => '',
        'childless' => false,
        'cache_domain' => 'core',
        'update_term_meta_cache' => true,
        'meta_query' => ''
    ];

    /**
     * Contains a list of items in the menu
     *
     * @var WP_Term|array
     */
    protected $items = [];

    public function __construct($menu = [])
    {
        parent::__construct($menu instanceof WP_Term ? $menu->to_array() : $menu);
    }

    /**
     * Retrieve the menu generated into a html string
     *
     * @param  $options
     *
     * @return false|object|string
     */
    public function generate(array $options = [])
    {
        return wp_nav_menu(array_replace_recursive($options, [
            'menu' => $this->term_id,
            'echo' => false
        ]));
    }

    /**
     * Retrieve menu items with option to build an hierarchical tree-structure
     *
     * @param bool $build_tree
     *
     * @return Collection
     */
    public function items(bool $build_tree = false): Collection
    {
        $items = [];
        if (false !== ($menu_items = wp_get_nav_menu_items($this->term_id))) {

            $items = array_map(function ($menu_item) {
                $menu_item->title = apply_filters( 'the_title', $menu_item->title, $this->term_id );

                return (array)$menu_item;
            }, $menu_items);

            $items = $build_tree ? $this->buildTree($items) : $items;
        }

        return new Collection($items);
    }

    /**
     * @param string $mark
     * @param bool   $build_tree
     *
     * @return Collection
     */
    public function markedItems(string $mark, $build_tree = false): Collection
    {
        return array_map(function ($item) use ($mark) {
            $classes = array_filter($item['classes']);
            $classes[] = $mark;
            $item['classes'] = $classes;

            return $item;
        }, $this->items($build_tree)->toArray());
    }

    /**
     * Render the menu
     *
     * @param $options
     *
     * @return void
     */
    public function render(array $options): void
    {
        wp_nav_menu(array_replace_recursive($options, [
            'menu' => $this->term_id,
            'echo' => true
        ]));
    }

    /**
     * Build an hierarchical tree out of an array of menu-items
     *
     * @param array $elements
     * @param int   $parentId
     *
     * @return Collection
     */
    protected function buildTree(array $elements = [], int $parentId = 0): Collection
    {
        $branch = [];

        foreach ($elements as $element) {
            if ((int)$element['menu_item_parent'] === $parentId) {
                $element['children'] = $this->buildTree($elements, $element['ID']);
                $branch[] = $element;
            }
        }

        return new Collection($branch);
    }

    /**
     * Retrieves all menus
     *
     * @param array $args Optional. Array of arguments passed on to {@see get_terms()}.
     *                    Default empty array.
     *
     * @return Collection
     */
    public static function all($args = [])
    {
        $menus = wp_get_nav_menus(array_replace_recursive(static::$defaults, $args));

        $menu_collection = [];
        if (\is_array($menus)) {
            $menu_collection = array_map(function (WP_Term $menu) {
                return new static($menu);
            }, $menus);
        }

        return new Collection($menu_collection);
    }

    /**
     * Results array of items from Collection or Arrayable.
     *
     * @param  mixed $items
     *
     * @return array
     */
    protected function getArrayableItems($items): array
    {
        if (\is_array($items)) {
            return $items;
        }
        if ($items instanceof WP_Term) {
            return array_change_key_case($items->to_array(), CASE_LOWER);
        }
        if ($items instanceof Arrayable) {
            return $items->toArray();
        }
        if ($items instanceof Jsonable) {
            return json_decode($items->toJson(), true);
        }

        return (array)$items;
    }
}
