<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use AbstractOcObject;
use Illuminate\Support\Arr;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Collection;
use OcArticle;

/**
 * PropertyParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base
 * @since   Infomaker\Everyware\Base\PropertyParser 1.0.0
 */
class PropertyParser
{

    /**
     * Contains project config
     *
     * @var PropertyMapConfig
     */
    protected $config;

    /**
     * @var array
     */
    private static $default_transforms = [
        'toSingleValue'
    ];

    public function __construct(PropertyMapConfig $config)
    {
        $this->config = $config;
    }

    public function fromOcObject(string $reference, AbstractOcObject $oc_object): PropertyObject
    {
        $properties = $this->extractProperties($oc_object->get_all_properties(),
            $this->config->getTypeProperty($reference));

        if ($oc_object instanceof OcArticle) {
            $properties = array_replace($properties, [
                'permalink' => $oc_object->get_permalink(),
                'post_id' => $oc_object->get_post()->ID
            ]);
        }

        return $this->getReferenceClass($reference, $properties);
    }

    public function fromSearch(string $reference, array $collection = []): Collection
    {
        return Collection::make($collection)->map(function ($object) use ($reference) {
            if ($this->validObject($object)) {
                return $this->fromOcObject($reference, $object);
            }

            return null;
        })->filter();
    }

    public function validObject($object): bool
    {
        return $object instanceof AbstractOcObject;
    }

    public function toBoolValue(array $property): bool
    {
        return filter_var($this->toSingleValue($property), FILTER_VALIDATE_BOOLEAN);
    }

    public function toDateValue(array $property): Date
    {
        return Date::createFromOcString($this->toSingleValue($property));
    }

    public function toMultiValue(array $property): array
    {
        return $property !== null ? (array)$property : [];
    }

    public function toSingleValue(array $property)
    {
        $value = $this->toMultiValue($property);

        return ! empty($value) ? array_shift($value) : '';
    }

    private function getReferenceClass(string $reference, array $properties = []): PropertyObject
    {
        $property_object = new GenericPropertyObject();
        $property_object->fill($properties);

        $class_name = Arr::get($this->config->getPropertyClassReference(), $reference, '');

        $class_instance = ! empty($class_name) ? new $class_name($property_object) : $property_object;

        if ( ! $class_instance instanceof PropertyObject) {
            NewRelicLog::error(sprintf('The reference class "%s" for propertyObject "%s" should extend "%s" and will therefor not be used.',
                $class_name, $reference, PropertyObject::class));

            return $property_object;
        }

        return $class_instance;
    }

    private function extractProperties(array $properties = [], array $mapped_properties = []): array
    {
        $extracted_properties = [];
        foreach ($mapped_properties as $property_map => $map_info) {
            $property = Arr::get($properties, strtolower($map_info['name']), []);
            $transforms = $this->getTransforms($map_info);
            if ($this->hasReference($map_info)) {
                $extracted_properties[$property_map] = $this->fromSearch($this->getReference($map_info), $property);
                continue;
            }
            foreach ($transforms as $transform) {
                $property = $this->$transform($property);
            }

            $extracted_properties[$property_map] = $this->hasReference($map_info) ? $this->fromSearch($this->getReference($map_info),
                $property) : $property;
        }

        return $extracted_properties;
    }

    private function hasReference($property_map): bool
    {
        return isset($property_map['propertyMapReference']);
    }

    private function getReference($property_map)
    {
        return $property_map['propertyMapReference'];
    }

    private function getTransforms(array $property_map = []): array
    {
        return Arr::get($property_map, 'transforms', static::$default_transforms);
    }
}
